import { Exception, JavaScriptException } from "../Exception";
import { app } from "../app";
export function errorToException(error) {
    if (error instanceof Exception) {
        return error;
    }
    else if (error instanceof Error) {
        return new JavaScriptException(error.message);
    }
    else {
        try {
            const errorMessage = JSON.stringify(error);
            return new JavaScriptException(errorMessage);
        }
        catch (e) {
            return new JavaScriptException("[Unknown Error]");
        }
    }
}
export function captureError(error, action, extra = {}) {
    if (process.env.NODE_ENV === "development") {
        console.error(`[framework] Error captured from [${action}]`, error);
    }
    const exception = errorToException(error);
    const errorStacktrace = error instanceof Error ? error.stack : undefined;
    const info = Object.assign(Object.assign({}, extra), { stacktrace: errorStacktrace });
    app.logger.exception(exception, info, action);
    app.sagaMiddleware.run(runUserErrorHandler, app.errorHandler, exception);
    return exception;
}
let isUserErrorHandlerRunning = false;
export function* runUserErrorHandler(handler, exception) {
    if (isUserErrorHandlerRunning)
        return;
    try {
        isUserErrorHandlerRunning = true;
        yield* handler(exception);
    }
    catch (e) {
        console.warn("[framework] Fail to execute user-defined error handler", e);
    }
    finally {
        isUserErrorHandlerRunning = false;
    }
}
//# sourceMappingURL=error-util.js.map